

from datetime import datetime

import pook
import pytest
from src.reservations import Reservation
from src.user import UserInfo

from tests.constants import (ALREADY_CHECKED_IN, CHECK_IN_PAST, DURATION,
                             EMPTY_JSON_RESPONSE, JSON_RESPONSE, START_TIME,
                             SUCCESSFUL_CHECK_IN, SUCCESSFUL_RESERVATION,
                             TOO_FAR_RESERVATION, UNSUCCESSFUL_RESERVATION)


def test_format_datetime():
    formatted = Reservation.format_datetime(datetime(2022, 8, 22, 11))
    assert formatted == "2022-08-22T11:00:00Z"
    formatted = Reservation.format_datetime(datetime(2022, 8, 22, 15))
    assert formatted == "2022-08-22T15:00:00Z"


def test_build_reservation_url():
    url = Reservation.build_list_reservations_url(datetime(2022, 8, 22, 11), datetime(2022, 8, 22, 19), 1)
    assert url == "https://api.robinpowered.com/v1.0/reservations/seats?before=2022-08-22T19:00:00Z&after=2022-08-22T11:00:00Z&seat_ids=1"
    url = Reservation.build_list_reservations_url(datetime(2022, 8, 22, 11, 22, 56), datetime(2022, 8, 22, 19, 14, 32), 1)
    assert url == "https://api.robinpowered.com/v1.0/reservations/seats?before=2022-08-22T19:00:00Z&after=2022-08-22T11:00:00Z&seat_ids=1"


@pook.on
def test_get_reservation_id():
    url = 'https://api.robinpowered.com/v1.0/reservations/seats?before=2022-08-22T19:00:00Z&after=2022-08-22T11:00:00Z&seat_ids=196962'
    pook.get(url, reply=200, response_json=JSON_RESPONSE)
    user_info = UserInfo("doe@factset.com", START_TIME, DURATION, 196962, 619521, "Etc/UTC")
    reservation = Reservation(user_info, datetime(2022, 8, 22))
    assert reservation._get_id() == "2254519933545219475"


@pook.on
def test_no_reservations():
    url = 'https://api.robinpowered.com/v1.0/reservations/seats?before=2022-08-23T19:00:00Z&after=2022-08-23T11:00:00Z&seat_ids=196962'
    pook.get(url, reply=200, response_json=EMPTY_JSON_RESPONSE)
    user_info = UserInfo("doe@factset.com", START_TIME, DURATION, 196962, 619521, "Etc/UTC")
    reservation = Reservation(user_info, datetime(2022, 8, 23))
    assert reservation._get_id() == "-1"


@pook.on
def test_successful_reservation():
    url, body = Reservation.build_reserve_request(
        datetime(2022, 9, 22, 11), datetime(2022, 9, 22, 19), 196962, 619521, "dbarone@factset.com", "Etc/UTC")
    pook.post(url, json=body, reply=200, response_json=SUCCESSFUL_RESERVATION)
    user_info = UserInfo("dbarone@factset.com", START_TIME, DURATION, 196962, 619521, "Etc/UTC")
    reservation = Reservation(user_info, datetime(2022, 9, 22))
    assert reservation._reserve() == "2254543828679656478"


@pook.on
def test_reservation_too_far_in_future():
    url, body = Reservation.build_reserve_request(
        datetime(2022, 9, 22, 11), datetime(2022, 9, 22, 19), 196962, 619521, "dbarone@factset.com", "Etc/UTC")
    pook.post(url, json=body, reply=400, response_json=TOO_FAR_RESERVATION)
    user_info = UserInfo("dbarone@factset.com", START_TIME, DURATION, 196962, 619521, "Etc/UTC")
    reservation = Reservation(user_info, datetime(2022, 9, 22))
    assert reservation._reserve() == "-1"


@pook.on
def test_unsuccessful_reservation():
    url, body = Reservation.build_reserve_request(
        datetime(2022, 9, 22, 11), datetime(2022, 9, 22, 19), 196962, 619521, "dbarone@factset.com", "Etc/UTC")
    pook.post(url, json=body, reply=400,
              response_json=UNSUCCESSFUL_RESERVATION)
    user_info = UserInfo("dbarone@factset.com", START_TIME, DURATION, 196962, 619521, "Etc/UTC")
    reservation = Reservation(user_info, datetime(2022, 9, 22))
    with pytest.raises(Exception, match=r"request .* failed with .* and err: .*"):
        reservation._reserve()


@pook.on
def test_check_in():
    url, body = Reservation.build_checkin_request(
        619521, "2259587448109532436")
    pook.put(url, json=body, reply=200,
             response_json=SUCCESSFUL_CHECK_IN)
    user_info = UserInfo("dbarone@factset.com", START_TIME, DURATION, 196962, 619521, "Etc/UTC")
    reservation = Reservation(user_info, datetime(2022, 8, 23))
    assert reservation._check_in(2259587448109532436) == ["2259587448109532436", "2022-08-26 15:19:16.111562"]


@pook.on
def test_check_in_past():

    url, body = Reservation.build_checkin_request(
        619521, "2254543828679656478")
    pook.put(url, json=body, reply=400,
             response_json=CHECK_IN_PAST)
    user_info = UserInfo("dbarone@factset.com", START_TIME, DURATION, 196962, 619521, "Etc/UTC")
    reservation = Reservation(user_info, datetime(2022, 8, 23))
    with pytest.raises(Exception, match=r"request .* failed with .* and err: .*") as e:
        reservation._check_in(2254543828679656478)
    expected_exception_message = "request https://api.robinpowered.com/v1.0/reservations/seats/2254543828679656478" \
        "/confirmation {'user_id': 619521} failed with 400 and err: You cannot confirm a seat reservation in the past."
    assert expected_exception_message == str(e.value)


@pook.on
def test_check_in_already_exists():
    url, body = Reservation.build_checkin_request(619521, "2254543828679656478")
    pook.put(url, json=body, reply=204000,
             response_json=ALREADY_CHECKED_IN)
    user_info = UserInfo("dbarone@factset.com", START_TIME, DURATION, 196962, 619521, "Etc/UTC")
    reservation = Reservation(user_info, datetime(2022, 8, 23))
    with pytest.raises(Exception, match=r"request .* failed with .* and err: .*") as e:
        reservation._check_in(2254543828679656478)

    expected_exception_message = "request https://api.robinpowered.com/v1.0/reservations/seats/2254543828679656478/" \
        "confirmation {'user_id': 619521} failed with 204000 and err: Cannot confirm desk with ID `2259583504062875574`. Confirmation already exists."
    assert expected_exception_message == str(e.value)



from datetime import datetime, timedelta
from typing import Dict, List

from src.reservations import Reservation


class Robin:
    def __init__(self, users_info: List):
        self.users_info = users_info

    def daterange(self, start_date, end_date):
        for n in range(int((end_date - start_date).days)):
            yield start_date + timedelta(n)

    def reserve(self, start: datetime, end: datetime = None) -> Dict:
        # there will always be only one seat_id for now
        results = {}
        for user_info in self.users_info:
            seat_id, reserver_id = user_info.seat_id, user_info.reserver_id
            results[reserver_id] = {}
            for current in self.daterange(start, end):
                if current.weekday() > 4:
                    print(f"skip weekends {current.strftime('%Y-%m-%d')}")
                    continue

                print(
                    f"make a reservation seat_id: {seat_id} date: {current.strftime(f'%Y-%m-%d')}")
                reservation = Reservation(user_info, current)
                if "-1" != reservation._get_id():
                    continue

                reservation_id = reservation._reserve()
                if "-1" == reservation_id:
                    print(f"reservation {reservation_id} too far in the future. stop going through dates")
                    break
                print(f"reservation successful with id {reservation_id}")
                results[reserver_id][current.strftime("%Y-%m-%d")] = True

        return results

    def check_in(self, current: datetime) -> Dict:
        results = {}
        for user_info in self.users_info:
            _, reserver_id = user_info.seat_id, user_info.reserver_id
            reservation = Reservation(user_info, current)
            reservation_id = reservation._get_id()
            if "-1" == reservation_id:
                print(f"no reservation to confirm for {reserver_id}")
                continue

            reservation._check_in(reservation_id)
            print(f"check in successful for {reservation_id}")
            results[reserver_id] = True

        return results



import email
import os

import requests


class UserInfo:

    USER_URL_TEMPLATE = "https://api.robinpowered.com/v1.0/users/{}"

    def __init__(self, email: str, stime: int, duration: int, sid: int, rid: int = None, tz: str = None):
        self.email = email
        self.seat_id = sid
        self.start_time = stime
        self.duration = duration
        self._token = os.getenv("ROBIN_AUTH_TOKEN")
        self._headers = {"Authorization": f"Access-Token {self._token}"}
        user_info = self._get_user_info() if not rid or not tz else None
        self.reserver_id = rid if rid else user_info['id']
        self.timezone = tz if tz else user_info['time_zone']

    @classmethod
    def build_user_url(cls, email: str) -> str:
        return cls.USER_URL_TEMPLATE.format(email)

    def _get_user_info(self):
        url = self.build_user_url(self.email)
        response = requests.get(url, headers=self._headers, verify=False)
        if response.status_code == 200:
            return response.json()['data']

        raise Exception(f"user {email} not found")



from datetime import datetime

import pook
from src.reservations import Reservation
from src.robin import Robin
from src.user import UserInfo

from tests.constants import (DURATION, EMPTY_JSON_RESPONSE, JSON_RESPONSE,
                             START_TIME, SUCCESSFUL_CHECK_IN,
                             SUCCESSFUL_RESERVATION)


@pook.on
def test_robin_reservation():
    users_info = [UserInfo("dbarone@factset.com", START_TIME, DURATION, 111, 619521, "Etc/UTC")]
    r = Robin(users_info)
    pook.get(Reservation.build_list_reservations_url(datetime(2022, 9, 22, 11), datetime(2022, 9, 22, 19), 111),
             reply=200, response_json=EMPTY_JSON_RESPONSE)
    pook.get(Reservation.build_list_reservations_url(datetime(2022, 9, 23, 11), datetime(2022, 9, 23, 19), 111),
             reply=200, response_json=EMPTY_JSON_RESPONSE)
    url, body = Reservation.build_reserve_request(
        datetime(2022, 9, 22, 11), datetime(2022, 9, 22, 19), 111, 619521, "dbarone@factset.com", "Etc/UTC")
    pook.post(url, json=body, reply=200, response_json=SUCCESSFUL_RESERVATION)
    url, body = Reservation.build_reserve_request(
        datetime(2022, 9, 23, 11), datetime(2022, 9, 23, 19), 111, 619521, "dbarone@factset.com", "Etc/UTC")
    pook.post(url, json=body,
              reply=200, response_json=SUCCESSFUL_RESERVATION)
    results = r.reserve(datetime(2022, 9, 22), datetime(2022, 9, 25))
    assert results == {619521: {'2022-09-22': True, '2022-09-23': True}}


@pook.on
def test_checkin_reservation():

    users_info = [UserInfo("dbarone@factset.com", START_TIME, DURATION, 111, 1, "Etc/UTC"),
                  UserInfo("doe@factset.com", START_TIME, DURATION, 222, 2, "Etc/UTC")]
    r = Robin(users_info)
    pook.get(Reservation.build_list_reservations_url(datetime(2022, 9, 22, 11), datetime(2022, 9, 22, 19), 111),
             reply=200, response_json=JSON_RESPONSE)
    pook.get(Reservation.build_list_reservations_url(datetime(2022, 9, 22, 11), datetime(2022, 9, 22, 19), 222),
             reply=200, response_json=EMPTY_JSON_RESPONSE)
    url, body = Reservation.build_checkin_request(
        1, "2254519933545219475")
    pook.put(url, json=body, reply=200,
             response_json=SUCCESSFUL_CHECK_IN)
    results = r.check_in(datetime(2022, 9, 22))
    assert results == {1: True}



START_TIME = 11
DURATION = 8

USER_FOUND_RESPONSE = {
    "meta": {
        "status_code": 200,
        "status": "OK",
        "message": "",
        "more_info": {},
        "errors": []
    },
    "data": {
        "id": 619521,
        "name": "Donato Barone",
        "family_name": "Barone",
        "given_name": "Donato",
        "slug": "donato-barone",
        "avatar": "https://static.robinpowered.com/reimagine/images/blahblah.jpg",
        "time_zone": "Etc/UTC",
        "created_at": "2019-12-03T19:42:46+0000",
        "updated_at": "2021-12-03T14:17:02+0000",
        "is_pending": False,
        "primary_email": {
            "email": "dbarone@factset.com",
            "is_verified": True
        },
        "working_hours": None
    }
}

USER_NOT_FOUND = {
    "meta": {
        "status_code": 404,
        "status": "NOT_FOUND",
        "message": "Object does not exist",
        "more_info": {},
        "errors": []
    },
    "data": {}
}

JSON_RESPONSE = {
    "meta": {
        "status_code": 200,
        "status": "OK",
        "message": "",
        "more_info": {},
        "errors": []
    },
    "data": [
        {
            "id": "2254519933545219475",
            "group_seat_reservation_id": None,
            "seat_id": 196962,
            "reserver_id": 619521,
            "type": "hoteled",
            "title": None,
            "start": {
                "date_time": "2022-08-22T11:00:00+0100",
                "time_zone": "Etc/UTC"
            },
            "end": {
                "date_time": "2022-08-22T19:00:00+0100",
                "time_zone": "Etc/UTC"
            },
            "recurrence": None,
            "series_id": None,
            "recurrence_id": None,
            "created_at": "2022-08-19T15:28:39+0000",
            "updated_at": "2022-08-19T15:28:39+0000",
            "reservee": {
                "email": "dbarone@factset.com",
                "user_id": 619521,
                "visitor_id": None,
                "participation_status": "not_responded"
            },
            "confirmation": None
        }
    ],
    "paging": {
        "page": 1,
        "per_page": 10,
        "has_next_page": False
    }
}

EMPTY_JSON_RESPONSE = {
    "meta": {
        "status_code": 200,
        "status": "OK",
        "message": "",
        "more_info": {},
        "errors": []
    },
    "data": [],
    "paging": {
        "page": 1,
        "per_page": 10,
        "has_next_page": False
    }
}

RESERVATION_BODY = {
    "type": "hoteled",
    "start": {
        "date_time": "2022-09-22T11:00:00Z",
        "time_zone": "Etc/UTC"
    },
    "end": {
        "date_time": "2022-09-22T19:00:00Z",
        "time_zone": "Etc/UTC"
    },
    "reservee": {
        "email": "dbarone@factset.com"
    },
    "reserver_id": 619521
}

SUCCESSFUL_RESERVATION = {
    "meta": {
        "status_code": 200,
        "status": "OK",
        "message": "",
        "more_info": {},
        "errors": []
    },
    "data": {
        "id": "2254543828679656478",
        "group_seat_reservation_id": None,
        "seat_id": 196962,
        "reserver_id": 619521,
        "type": "hoteled",
        "title": None,
        "start": {
            "date_time": "2022-08-23T11:00:00+0100",
            "time_zone": "Etc/UTC"
        },
        "end": {
            "date_time": "2022-08-23T19:00:00+0100",
            "time_zone": "Etc/UTC"
        },
        "recurrence": None,
        "series_id": None,
        "recurrence_id": None,
        "created_at": "2022-08-19T16:16:08+0000",
        "updated_at": "2022-08-19T16:16:08+0000",
        "reservee": {
            "email": "dbarone@factset.com",
            "user_id": 619521,
            "visitor_id": None,
            "participation_status": "not_responded"
        },
        "confirmation": None
    }
}

TOO_FAR_RESERVATION = {
    "meta": {
        "status_code": 400,
        "status": "BAD_REQUEST",
        "message": "The reservation is too far in the future. Reservations for seat `196962` cannot be more than 14 days in advance.",
        "more_info": {},
        "errors": [
            {
                "domain": "scheduling",
                "reason": "seat_booking_policy_violation",
                "message": "The reservation is too far in the future. Reservations for seat `196962` cannot be more than 14 days in advance.",
                "details": {
                    "policy": "advanced_booking_threshold",
                    "booking_threshold": "P14D"
                }
            }
        ]
    },
    "data": {}
}
UNSUCCESSFUL_RESERVATION = {
    "meta": {
        "status_code": 400,
        "status": "BAD_REQUEST",
        "message": "Something else. Reservations for seat `196962`.",
        "more_info": {},
        "errors": [
            {
                "domain": "scheduling",
                "reason": "seat_booking_policy_violation",
                "message": "Something else. Reservations for seat `196962`.",
                "details": {
                    "policy": "another_error",
                    "booking_threshold": "P14D"
                }
            }
        ]
    },
    "data": {}
}

ALREADY_CHECKED_IN = {
    "meta": {
        "status_code": 400,
        "status": "BAD_REQUEST",
        "message": "Cannot confirm desk with ID `2259583504062875574`. Confirmation already exists.",
        "more_info": {},
        "errors": [
            {
                "domain": "scheduling",
                "reason": "event_already_confirmed",
                "message": "Cannot confirm desk with ID `2259583504062875574`. Confirmation already exists."
            }
        ]
    },
    "data": {}
}

CHECK_IN_PAST = {
    "meta": {
        "status_code": 400,
        "status": "BAD_REQUEST",
        "message": "You cannot confirm a seat reservation in the past.",
        "more_info": {},
        "errors": [
            {
                "domain": "scheduling",
                "reason": "too_late_to_confirm_event",
                "message": "You cannot confirm a seat reservation in the past."
            }
        ]
    },
    "data": {}
}

SUCCESSFUL_CHECK_IN = {
    "meta": {
        "status_code": 200,
        "status": "OK",
        "message": "",
        "more_info": {},
        "errors": []
    },
    "data": {
        "seat_reservation_id": "2259587448109532436",
        "device_id": None,
        "user_id": 619521,
        "confirmed_at": {
            "date": "2022-08-26 15:19:16.111562",
            "timezone_type": 3,
            "timezone": "UTC"
        }
    }
}



import pook
import pytest
from src.user import UserInfo

from tests.constants import (DURATION, START_TIME, USER_FOUND_RESPONSE,
                             USER_NOT_FOUND)


@pook.on
def test_user():
    pook.get(UserInfo.build_user_url("dbarone@factset.com"),
             reply=200, response_json=USER_FOUND_RESPONSE)
    user = UserInfo("dbarone@factset.com", START_TIME, DURATION, 1)
    assert user.reserver_id == 619521

    pook.get(UserInfo.build_user_url("doe@factset.com"),
             reply=404, response_json=USER_NOT_FOUND)
    with pytest.raises(Exception, match=r"user .* not found"):
        user = UserInfo("doe@factset.com", START_TIME, DURATION, 1)



from setuptools import find_packages, setup

setup(
    name='robin-powered-bot',
    version='0.0.1',
    description='Provides a set of utilities to simplify communication with robin-powered API and more easily automate bookings.',
    url='https://github.com/donatobarone/robin-powered-bot',
    author='Donato Barone',
    author_email='eng.donato.barone@gmail.com',
    packages=find_packages(),
    install_requires=[
        'requests'
    ], zip_safe=False)



import os
from datetime import datetime, timedelta
from typing import List

import requests

from src.user import UserInfo


class Reservation:

    LIST_RESERVATIONS_URL_TEMPLATE = "https://api.robinpowered.com/v1.0/reservations/seats?before={}&after={}&seat_ids={}"
    SET_RESERVATION_URL_TEMPLATE = "https://api.robinpowered.com/v1.0/seats/{}/reservations"
    CHECKIN_URL_TEMPLATE = "https://api.robinpowered.com/v1.0/reservations/seats/{}/confirmation"
    EMPTY_RESERVATION_ID = "-1"

    def __init__(self, user_info: UserInfo, rdate: datetime = None):
        self._user_info = user_info
        current_time = rdate if rdate else datetime.utcnow()
        self._reservation_date_start = current_time.replace(hour=user_info.start_time, minute=0, second=0, microsecond=0)
        self._reservation_date_end = self._reservation_date_start + timedelta(hours=user_info.duration)
        token = os.getenv("ROBIN_AUTH_TOKEN", "")
        self._headers = {"Authorization": f"Access-Token {token}"}

    @classmethod
    def format_datetime(cls, date: datetime) -> str:
        return date.strftime('%Y-%m-%dT%H:00:00Z')

    @classmethod
    def build_list_reservations_url(cls, sdate: datetime, edate: datetime, sid: int):
        return cls.LIST_RESERVATIONS_URL_TEMPLATE.format(
            cls.format_datetime(edate),
            cls.format_datetime(sdate), sid)

    @classmethod
    def build_reserve_request(cls, sdate: datetime, edate: datetime, sid: int, rid: int, email: str, timezone: str) -> List:
        url = cls.SET_RESERVATION_URL_TEMPLATE.format(sid)
        body = {'type': 'hoteled',
                "start": {
                    "date_time": cls.format_datetime(sdate),
                    "time_zone": timezone
                },
                "end": {
                    "date_time": cls.format_datetime(edate),
                    "time_zone": timezone
                },
                "reservee": {
                    "email": email
                },
                "reserver_id": rid}
        return [url, body]

    @classmethod
    def build_checkin_request(cls, reserver_id: int, reservation_id: str) -> List:
        url = cls.CHECKIN_URL_TEMPLATE.format(reservation_id)
        body = {"user_id": reserver_id}
        return [url, body]

    def _get_id(self) -> str:
        url = self.build_list_reservations_url(self._reservation_date_start, self._reservation_date_end, self._user_info.seat_id)
        response = requests.get(url, headers=self._headers, verify=False)
        reservations = response.json()['data']
        if len(reservations) == 0:
            return self.EMPTY_RESERVATION_ID
        if len(reservations) > 1:
            raise Exception("should only contain 1 reservation")
        reservation = reservations[0]
        if reservation['confirmation'] is not None:
            return Exception("already checked in")
        return reservation['id']

    def _reserve(self) -> str:
        url, body = self.build_reserve_request(self._reservation_date_start, self._reservation_date_end,
                                               self._user_info.seat_id, self._user_info.reserver_id, self._user_info.email, self._user_info.timezone)
        response = requests.post(
            url, json=body, headers=self._headers, verify=False)
        if response.status_code == 200:
            return response.json()['data']['id']
        if response.status_code == 400:
            meta_response = response.json()['meta']
            try:
                errors = meta_response['errors']
                for error in errors:
                    if error['details']['policy'] == "advanced_booking_threshold":
                        return self.EMPTY_RESERVATION_ID
            except KeyError:
                pass
        raise Exception(
            f"request {url} {body} failed with {response.status_code} and err: {response.json()['meta']['message']}")

    def _check_in(self, reservation_id: int):
        url, body = self.build_checkin_request(self._user_info.reserver_id, reservation_id)
        response = requests.put(
            url, json=body, headers=self._headers, verify=False)
        if response.status_code == 200:
            data_response = response.json()['data']
            return [data_response['seat_reservation_id'], data_response['confirmed_at']['date']]
        raise Exception(
            f"request {url} {body} failed with {response.status_code} and err: {response.json()['meta']['message']}")
